"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useConnectForm = exports.handleConnectionFormFieldUpdate = void 0;
const react_1 = require("react");
const mongodb_connection_string_url_1 = __importDefault(require("mongodb-connection-string-url"));
const default_connection_1 = require("../constants/default-connection");
const connect_form_warnings_1 = require("../utils/connect-form-warnings");
const connect_form_errors_1 = require("../utils/connect-form-errors");
const get_next_host_1 = require("../utils/get-next-host");
const default_connection_2 = require("../constants/default-connection");
const markable_form_fields_1 = require("../constants/markable-form-fields");
const check_for_invalid_character_in_host_1 = require("../utils/check-for-invalid-character-in-host");
const connection_string_schema_1 = require("../utils/connection-string-schema");
function connectFormReducer(state, action) {
    switch (action.type) {
        case 'set-connection-string-error':
            return {
                ...state,
                connectionStringInvalidError: action.errorMessage,
            };
        case 'set-connection-string-url':
            return {
                ...state,
                connectionStringUrl: action.connectionStringUrl,
                connectionStringInvalidError: null,
            };
        case 'set-connection-string-state':
            return {
                ...state,
                ...action.newState,
            };
        case 'hide-error':
            return {
                ...state,
                errors: [...state.errors].splice(action.errorIndex, 1),
            };
        case 'hide-warning':
            return {
                ...state,
                warnings: [...state.warnings].splice(action.warningIndex, 1),
            };
    }
}
function buildStateFromConnectionInfo(initialConnectionInfo) {
    let connectionStringInvalidError = null;
    let connectionStringUrl = new mongodb_connection_string_url_1.default(default_connection_1.defaultConnectionString);
    try {
        connectionStringUrl = new mongodb_connection_string_url_1.default(initialConnectionInfo.connectionOptions.connectionString);
    }
    catch (error) {
        connectionStringInvalidError = error.message;
    }
    const connectionOptions = {
        ...initialConnectionInfo.connectionOptions,
    };
    return {
        errors: connect_form_errors_1.getConnectFormErrors(connectionOptions),
        warnings: connect_form_warnings_1.getConnectFormWarnings(connectionOptions),
        connectionStringInvalidError,
        connectionStringUrl,
        connectionOptions,
    };
}
function handleUpdateHost({ action, connectionStringUrl, connectionOptions, }) {
    const { newHostValue, hostIndex } = action;
    try {
        check_for_invalid_character_in_host_1.checkForInvalidCharacterInHost(newHostValue, connectionStringUrl.isSRV);
        if (connectionStringUrl.hosts.length === 1 && newHostValue === '') {
            throw new Error('Host cannot be empty. The host is the address hostname, IP address, or UNIX domain socket where the mongodb instance is running.');
        }
        const updatedConnectionString = connectionStringUrl.clone();
        updatedConnectionString.hosts[hostIndex] = newHostValue || '';
        const newConnectionStringUrl = new mongodb_connection_string_url_1.default(updatedConnectionString.toString());
        return {
            connectionStringUrl: newConnectionStringUrl,
            connectionOptions: {
                ...connectionOptions,
                connectionString: newConnectionStringUrl.toString(),
            },
            errors: [],
        };
    }
    catch (err) {
        return {
            connectionStringUrl,
            connectionOptions: {
                ...connectionOptions,
                connectionString: connectionStringUrl.toString(),
            },
            errors: [
                {
                    fieldName: markable_form_fields_1.MARKABLE_FORM_FIELD_NAMES.HOSTS,
                    hostIndex,
                    message: err.message,
                },
            ],
        };
    }
}
function handleConnectionFormFieldUpdate({ action, connectionStringUrl, connectionOptions, }) {
    const updatedConnectionStringUrl = connectionStringUrl.clone();
    switch (action.type) {
        case 'add-new-host': {
            const { hostIndexToAddAfter } = action;
            const newHost = get_next_host_1.getNextHost(updatedConnectionStringUrl.hosts, hostIndexToAddAfter);
            updatedConnectionStringUrl.hosts.splice(hostIndexToAddAfter + 1, 0, newHost);
            if (updatedConnectionStringUrl.searchParams.get('directConnection')) {
                updatedConnectionStringUrl.searchParams.delete('directConnection');
            }
            return {
                connectionStringUrl: updatedConnectionStringUrl,
                connectionOptions: {
                    ...connectionOptions,
                    connectionString: updatedConnectionStringUrl.toString(),
                },
                errors: [],
            };
        }
        case 'remove-host': {
            const { hostIndexToRemove } = action;
            updatedConnectionStringUrl.hosts.splice(hostIndexToRemove, 1);
            if (updatedConnectionStringUrl.hosts.length === 1 &&
                !updatedConnectionStringUrl.hosts[0]) {
                updatedConnectionStringUrl.hosts[0] = `${default_connection_2.defaultHostname}:${default_connection_2.defaultPort}`;
            }
            return {
                connectionStringUrl: updatedConnectionStringUrl,
                connectionOptions: {
                    ...connectionOptions,
                    connectionString: updatedConnectionStringUrl.toString(),
                },
                errors: [],
            };
        }
        case 'update-host': {
            return handleUpdateHost({
                action,
                connectionStringUrl,
                connectionOptions,
            });
        }
        case 'update-direct-connection': {
            const { isDirectConnection } = action;
            if (isDirectConnection) {
                updatedConnectionStringUrl.searchParams.set('directConnection', 'true');
            }
            else if (updatedConnectionStringUrl.searchParams.get('directConnection')) {
                updatedConnectionStringUrl.searchParams.delete('directConnection');
            }
            return {
                connectionStringUrl: updatedConnectionStringUrl,
                connectionOptions: {
                    ...connectionOptions,
                    connectionString: updatedConnectionStringUrl.toString(),
                },
                errors: [],
            };
        }
        case 'update-connection-schema': {
            const { isSrv } = action;
            try {
                const newConnectionStringUrl = connection_string_schema_1.tryUpdateConnectionStringSchema(connectionStringUrl, isSrv);
                return {
                    connectionStringUrl: newConnectionStringUrl,
                    connectionOptions: {
                        ...connectionOptions,
                        connectionString: newConnectionStringUrl.toString(),
                    },
                    errors: [],
                };
            }
            catch (err) {
                return {
                    connectionStringUrl,
                    connectionOptions: {
                        ...connectionOptions,
                        connectionString: connectionStringUrl.toString(),
                    },
                    errors: [
                        {
                            fieldName: markable_form_fields_1.MARKABLE_FORM_FIELD_NAMES.IS_SRV,
                            message: `Error updating connection schema: ${err.message}`,
                        },
                    ],
                };
            }
        }
    }
}
exports.handleConnectionFormFieldUpdate = handleConnectionFormFieldUpdate;
function useConnectForm(initialConnectionInfo) {
    const [state, dispatch] = react_1.useReducer(connectFormReducer, initialConnectionInfo, buildStateFromConnectionInfo);
    react_1.useEffect(() => {
        const { errors, warnings, connectionStringInvalidError, connectionStringUrl, connectionOptions, } = buildStateFromConnectionInfo(initialConnectionInfo);
        dispatch({
            type: 'set-connection-string-state',
            newState: {
                errors,
                warnings,
                connectionStringInvalidError,
                connectionStringUrl,
                connectionOptions,
            },
        });
    }, [initialConnectionInfo]);
    function setConnectionStringUrl(connectionStringUrl) {
        dispatch({
            type: 'set-connection-string-url',
            connectionStringUrl,
        });
    }
    function updateConnectionFormField(action) {
        const { connectionOptions, connectionStringUrl, errors: formFieldErrors, } = handleConnectionFormFieldUpdate({
            action,
            connectionOptions: state.connectionOptions,
            connectionStringUrl: state.connectionStringUrl,
        });
        dispatch({
            type: 'set-connection-string-state',
            newState: {
                connectionStringInvalidError: null,
                errors: [
                    ...formFieldErrors,
                    ...connect_form_errors_1.getConnectFormErrors(connectionOptions),
                ],
                warnings: connect_form_warnings_1.getConnectFormWarnings(connectionOptions),
                connectionStringUrl,
                connectionOptions,
            },
        });
    }
    return [
        state,
        {
            setConnectionStringError: (errorMessage) => {
                dispatch({
                    type: 'set-connection-string-error',
                    errorMessage,
                });
            },
            setConnectionStringUrl,
            hideError: (errorIndex) => {
                dispatch({
                    type: 'hide-error',
                    errorIndex,
                });
            },
            hideWarning: (warningIndex) => {
                dispatch({
                    type: 'hide-warning',
                    warningIndex,
                });
            },
            updateConnectionFormField,
        },
    ];
}
exports.useConnectForm = useConnectForm;
//# sourceMappingURL=use-connect-form.js.map